==> multi-ajax-0.2.0.tgz: package/src/utils.ts <==
import hash from 'object-hash'

export function isAbsoluteURL(url: string) {
  // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
  // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
  // by any combination of letters, digits, plus, period, or hyphen.
  return /^([a-z][a-z\d+-.]*:)?\/\//i.test(url);
}

export function combineURLs(baseURL: string, relativeURL?: string) {
  return relativeURL
    ? `${baseURL.replace(/\/+$/, '')}/${relativeURL.replace(/^\/+/, '')}`
    : baseURL;
}

export function buildFullPath(baseURL: string, requestedURL: string): string {
  if (baseURL && !isAbsoluteURL(requestedURL)) {
    return combineURLs(baseURL, requestedURL);
  }
  return requestedURL;
}

type DeepCopy<T> = T extends object
    ? T extends Array<infer Item>
      ? DeepCopy<Item>[]
      : {
          [K in keyof T]: T[K] extends object
            ? DeepCopy<T[K]>
            : T[K]
        }
    : T

export function deepCopy<T>(obj: T): DeepCopy<T> {
  let target: any;
  if (typeof obj === 'object' && obj !== null) {
    if (Array.isArray(obj)) {
      target = [];
      obj.forEach((elem) => target.push(deepCopy(elem)));
    } else {
      target = {};
      Object.keys(obj).forEach((key) => {
        // @ts-ignore
        target[key] = deepCopy(obj[key]);
      });
    }
  } else {
    target = obj;
  }
  return target;
}

export function getType(obj: any): string {
  const type = typeof obj;
  if (type !== 'object') return type;

  return Object.prototype.toString.call(obj)
    .replace(/^\[object\s+(\S+)\]$/, '$1')
    .toLowerCase();
}


type NormalizedConfig = {
  params?: Record<string, any>
  data?: Record<string, any>
  url?: string
  baseURL?: string
  method?: string
}

/**
 * 获取请求的标识，相同的请求配置有相同的标识
 * @param config 经过标准化的ajax请求配置
 * @returns 该请求的hash key
 */
export function generateRequestHash(config: NormalizedConfig) {
  const { url = '', method = '', params = {}, data = {}, baseURL = '' } = config
  return hash({
    ...params,
    ...data,
    baseURL,
    url,
    method
  })
}

==> multi-ajax-0.2.3.tgz: package/src/utils.ts <==
import hash from 'object-hash'

export function isAbsoluteURL(url: string) {
  // A URL is considered absolute if it begins with "<scheme>://" or "//" (protocol-relative URL).
  // RFC 3986 defines scheme name as a sequence of characters beginning with a letter and followed
  // by any combination of letters, digits, plus, period, or hyphen.
  return /^([a-z][a-z\d+-.]*:)?\/\//i.test(url);
}

export function combineURLs(baseURL: string, relativeURL?: string) {
  return relativeURL
    ? `${baseURL.replace(/\/+$/, '')}/${relativeURL.replace(/^\/+/, '')}`
    : baseURL;
}

export function buildFullPath(baseURL: string, requestedURL: string): string {
  if (baseURL && !isAbsoluteURL(requestedURL)) {
    return combineURLs(baseURL, requestedURL);
  }
  return requestedURL;
}

type DeepCopy<T> = T extends object
    ? T extends Array<infer Item>
      ? DeepCopy<Item>[]
      : {
          [K in keyof T]: T[K] extends object
            ? DeepCopy<T[K]>
            : T[K]
        }
    : T

export function deepCopy<T>(obj: T): DeepCopy<T> {
  let target: any;
  if (typeof obj === 'object' && obj !== null) {
    if (Array.isArray(obj)) {
      target = [];
      obj.forEach((elem) => target.push(deepCopy(elem)));
    } else {
      target = {};
      Object.keys(obj).forEach((key) => {
        target[key] = deepCopy((obj as Record<string, any>)[key]);
      });
    }
  } else {
    target = obj;
  }
  return target;
}

export function getType(obj: any): string {
  const type = typeof obj;
  if (type !== 'object') return type;

  return Object.prototype.toString.call(obj)
    .replace(/^\[object\s+(\S+)\]$/, '$1')
    .toLowerCase();
}


type NormalizedConfig = {
  params?: Record<string, any>
  data?: Record<string, any>
  url?: string
  baseURL?: string
  method?: string
}

/**
 * 获取请求的标识，相同的请求配置有相同的标识
 * @param config 经过标准化的ajax请求配置
 * @returns 该请求的hash key
 */
export function generateRequestHash(config: NormalizedConfig) {
  const { url = '', method = '', params = {}, data = {}, baseURL = '' } = config
  return hash({
    ...params,
    ...data,
    baseURL,
    url,
    method
  })
}
